#!/usr/bin/env python
# -- coding: utf-8

#       Copyright 2007 Manuel Vazquez Acosta <mva.led@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

"""Linkbot.py

$Id: linkbot.py 478 2007-12-10 16:21:38Z manu $

Linkbot is design to read a page a extract all *outgoing* links.

The term outgoing is used in loose way: Currently we only
grab ALL of the hyperlinks (tag A with an HREF), despite its
"visibility". We think this is the "general" good way of doing
things.

All comments about this policy can be found (in spanish) in
http://wiki.prod.uci.cu/index.php/Linkbot
"""

from BeautifulSoup import BeautifulSoup, BeautifulStoneSoup

import re
from urlparse import urljoin, urldefrag, urlparse, urlunparse
import unicodedata

class Linkbot(BeautifulSoup):
    """Linkbot/$Rev: 478 $

    Given an (X)HTML page returns its outgoing
    links.

    Usage:

    linkbot = Linkbot(urlbase, htmldata)
    linkbot.getLinks()
    """

    def __init__(self, urlbase, htmldata = "", documentEncoding=None):
        # Linkbot must be provided with base URI
        assert urlbase != None and urlbase != ''

        #Tracks base URLs so we can conform to what's
        #stated in XML-BASE W3C Recommendation
        #http://www.w3.org/TR/2001/REC-xmlbase-20010627/
        #
        #Note: We DO NOT parse DTD so, default xml:base
        #attributes are not correctly handled!
        self.__urlbase_stack_ = [urlbase]
        self.__links_ = []

        print "Extracting links from %s" % urlbase
        BeautifulSoup.__init__(self, markup=htmldata, fromEncoding=documentEncoding)


    def do_base(self, attrs):
        """BASE tag support"""
        nattrs = {}
        for which in attrs:
            nattrs[which[0]]=which[1]

        if nattrs.has_key("href"):
            if isinstance(nattrs["href"], unicode):
                self.__urlbase_stack_.append(nattrs["href"].decode())
            else:
                self.__urlbase_stack_.append(nattrs["href"])

    def popTag(self):
        # If the current tag (the tag we pop)
        # has an xml:base attr we pop __urlbase_stack_
        # This is necessary in order to properly
        # handle nested xml:base
        if self.currentTag.has_key("xml:base"):
            self.__urlbase_stack_.pop()

        BeautifulSoup.popTag(self)

    def start_a(self, attrs):
        # Transform attrs to a dict
        nattrs = {}
        for which in attrs:
            nattrs[which[0]]=which[1]

        # Support for xml:base attribute
        if nattrs.has_key("xml:base"):
            urlbase = nattrs["xml:base"]
        else:
            urlbase = self.__urlbase_stack_[-1]

        # If this is a link (i.e. has an href
        # attribute) we grab it
        if nattrs.has_key("href"):
            newURI = self._addurls(urlbase, nattrs["href"])
            if not newURI in self.__links_:
                self.__links_.append(newURI)



    def pushTag(self, tag):
        # Checks whether this tag has an "xml:base"
        # attribute, this is necessary in order
        # to properly stack nested xml:base
        if tag.has_key("xml:base"):
            self.__urlbase_stack_.append(tag["xml:base"])

        BeautifulSoup.pushTag(self, tag)

    def getLinks(self):
        return  self.__links_

    def _addurls(self, baseurl, refurl):
        res = urljoin(baseurl, refurl, 0)

        # If too many ../ are placed in refurl, res may result in something
        # like: http://www.domain.org/../../path
        # So we should strip those remaining ../
        parsed = urlparse(res)
        pattern = re.compile(r'^(/..)+(/.*)$')
        Match = pattern.match(parsed[2])
        if Match:
            parsed = parsed[0:2] + (Match.group(2),) + parsed[3:]
            res = urlunparse(parsed)


        # It seems that allow_fragment=0 does nothing,
        # so we do it ourselves
        return urldefrag(res)[0]

def __main_():
    import sys, os, getopt

    opts = getopt.gnu_getopt(sys.argv[1:], "u:i:", ["url-base=", "input-file="])

    if len(opts[0]) != 2:
        raise getopt.GetoptError("2 params")

    urlbase = None
    inputfile = None

    for opt in opts[0]:
        if opt[0] == '-u' or opt[0]=='--url-base':
            urlbase = opt[1]

        if opt[0] == '-i' or opt[0] == '--input-file':
            inputfile = opt[1]

    fh = open(inputfile)
    html = fh.read()

    linkbot = Linkbot(urlbase, html)
    for which in linkbot.getLinks():
        print which


if __name__ == "__main__":
    __main_()
