#!/usr/bin/env python
# -- coding: latin-1

#       Copyright 2007 Manuel Vazquez Acosta <mva.led@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

"""Guglebot bootstrap

This script is just a wrapper around guglebot.


Usage:
bot.py [options]

Options can be:
    --run-for=secs
    --run-for-minutes=min

            These will tell bot.py how much time it will be running.
            --run-for-minutes will prevail if both are set.
            default: bot.py will run for 1 hour

    --steps=n
            This will instruct bot.py to report the status of the
            Basic Knowledge Base n times. bot.py will report the BKB
            at intervals during the whole run.
            default: 0, ie. No reporting, BKB status will be reported
            at the end of the run. However, WorkerBots may print some
            messages.

    --url-filters=file
            Read regular expressions in file, and only
            fetch URLs matching one of those expressions

    --feed=file
            This file is used at first to feed the spider in order
            start at some point in the Web.
            Notice that only URLs that matches filters provided by
            --url-filters are considered

    --threads=n
            Set the number of workerbots. Workerbots do the real
            work. The default is 3. Set at your own risk.

    --niceness=n
            Set how nice worketbots would be with the webservers
            they are requesting pages from. The higher n, the nicer.
            Default is 2. Set this to 0 to be bad.

    Proxy support:
        export http_proxy=http://[user[:password]@]host[:port]/
        That's it!
"""
from gugle.bot.guglebot import Guglebot

from threading import Timer
import os.path
import re

class Reporter:
    def __init__(self, bot, step, iterations):
        self.__bot = bot
        self.__step = step
        self.__iter = iterations
        self.__count = 0
        if iterations > 0:
            self.setTimer(self.report)
        else:
            self.setTimer(bot.kill)

    def report(self):
        self.__count = self.__count + 1
        self.__bot.report()
        if self.__count < self.__iter:
            self.setTimer(self.report)
        else:
            self.setTimer(self.__bot.kill)

    def cancel(self):
        if self.reporter:
            self.reporter.cancel()

    def setTimer(self, func):
        self.reporter = Timer(self.__step, func)
        self.reporter.start()

ConfigLineRegExp = re.compile(r'^\s*(allow|deny)\s+(.*)$')


def main():
    import os

    # Ensure a var directory for our storage
    if not os.path.exists('var'):
        os.mkdir('var')
    else:
        assert os.path.isdir('var'), "There's a file called var, we need a directory with that name"

    bot = Guglebot()

    #TODO: There's a saner way to do this.
    import getopt,sys
    try:
        options = [
            "run-for=",
            "run-for-minutes=",
            "step=",
            "url-filters=",
            "feed=",
            "threads=",
            "niceness="
        ]
        opts = getopt.getopt(sys.argv[1:], "", options)
    except getopt.GetoptError:
        print __doc__
        sys.exit(0)

    easy_opts = {}
    for opt in opts[0]:
        easy_opts[opt[0]] = opt[1]

    if easy_opts.has_key("--niceness"):
        try:
            niceness = int(easy_opts["--niceness"])
            from guglebot import workerbot
            workerbot.NICENESS = niceness
        except:
            raise

    if easy_opts.has_key("--threads"):
        try:
            threads = int(easy_opts["--threads"])
            from gugle.bot import guglebot
            guglebot.MAX_AMOUNT_OF_WORKERS = threads
        except:
            raise

    if easy_opts.has_key("--run-for-minutes"):
        MAX_BOT_RUNTIME = int(easy_opts["--run-for-minutes"])*60
        step_weight = 60
    elif easy_opts.has_key("--run-for"):
        MAX_BOT_RUNTIME = int(easy_opts["--run-for"])
        step_weight = 1
    else:
        MAX_BOT_RUNTIME = 3600 # 1 hour by default
        step_weight = 1

    if easy_opts.has_key("--url-filters") and os.path.isfile(easy_opts["--url-filters"]):
        file = open(easy_opts["--url-filters"])
        for line in file:
            Match = ConfigLineRegExp.match(line[:-1])
            if Match:
                print "Adding URL filter %s, with %s policy" % (Match.group(2), Match.group(1))
                bot.addURLFilter(Match.group(2), Match.group(1))

    if easy_opts.has_key("--feed") and os.path.isfile(easy_opts["--feed"]):
        file = open(easy_opts["--feed"])
        for line in file:
            bot.put(line[:-1])  # This is needed to strip \n last char

    if easy_opts.has_key("--step"):
        step = int(easy_opts["--step"]) * step_weight
        reporter = Reporter(bot, step, int((MAX_BOT_RUNTIME-step_weight)/step))
    else:
        reporter = Timer(MAX_BOT_RUNTIME, bot.kill)
        reporter.start()

    try:
        bot.run()
    except KeyboardInterrupt:
        reporter.cancel()


if __name__ == "__main__":
    main()
